/*******************************************************************************
* File Name: P15_5_Reconnect.c  
* Version 1.70
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "P15_5_Reconnect.h"


/*******************************************************************************
* Function Name: P15_5_Reconnect_Write
********************************************************************************
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  void 
*  
*******************************************************************************/
void P15_5_Reconnect_Write(uint8 value) 
{
    uint8 staticBits = P15_5_Reconnect_DR & ~P15_5_Reconnect_MASK;
    P15_5_Reconnect_DR = staticBits | ((value << P15_5_Reconnect_SHIFT) & P15_5_Reconnect_MASK);
}


/*******************************************************************************
* Function Name: P15_5_Reconnect_SetDriveMode
********************************************************************************
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to this drive mode.
*
* Return: 
*  void
*
*******************************************************************************/
void P15_5_Reconnect_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(P15_5_Reconnect_0, mode);
}


/*******************************************************************************
* Function Name: P15_5_Reconnect_Read
********************************************************************************
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  void 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro P15_5_Reconnect_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 P15_5_Reconnect_Read(void) 
{
    return (P15_5_Reconnect_PS & P15_5_Reconnect_MASK) >> P15_5_Reconnect_SHIFT;
}


/*******************************************************************************
* Function Name: P15_5_Reconnect_ReadDataReg
********************************************************************************
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  void 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 P15_5_Reconnect_ReadDataReg(void) 
{
    return (P15_5_Reconnect_DR & P15_5_Reconnect_MASK) >> P15_5_Reconnect_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(P15_5_Reconnect_INTSTAT) 

    /*******************************************************************************
    * Function Name: P15_5_Reconnect_ClearInterrupt
    ********************************************************************************
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  void 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 P15_5_Reconnect_ClearInterrupt(void) 
    {
        return (P15_5_Reconnect_INTSTAT & P15_5_Reconnect_MASK) >> P15_5_Reconnect_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */ 
