/*******************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
*   Enumerates as a Virtual Com port between a host and the eS-WiFi Module.
*   Passes data between the host and eS-WiFi via the Virtual Com port.
*   
*  To test project:
*   1. Build the project and program the hex file on to the target device.
*   2. Select 3.3V in SW3 and plug-in power to the CY8CKIT-001
*   3. Connect USB cable from the computer to the CY8CKIT-001.
*   4. Select the USB_UART.inf file from the project directory, as the driver 
*      for this example once Windows asks for it.
*   5. Open "Device Manager" and note the COM port number for "Example Project"
*      device.
*   6. Open "HyperTerminal" or some other comm port application and make 
*      new connection to noted COM port
*   7. Type '?' to see a list of eS-WiFi commands
*
* Related Document:
*  Universal Serial Bus Specification Revision 2.0 
*  Universal Serial Bus Class Definitions for Communications Devices 
*  Revision 1.2
*
********************************************************************************
* Copyright 2012, Cypress Semiconductor Corporation. All rights reserved.
* This software is owned by Cypress Semiconductor Corporation and is protected
* by and subject to worldwide patent and copyright laws and treaties.
* Therefore, you may use this software only as provided in the license agreement
* accompanying the software package from which you obtained this software.
* CYPRESS AND ITS SUPPLIERS MAKE NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* WITH REGARD TO THIS SOFTWARE, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT,
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*******************************************************************************
* Inventek System, Inc.
* 1. Disclaimer of Warranty.
* THERE IS NO WARRANTY FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. 
* EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES 
* PROVIDE THE SOFTWARE “AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR 
* IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
* AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND 
* PERFORMANCE OF THE SOFTWARE IS WITH YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, 
* YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
* 
* 2. Limitation of Liability.
* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY 
* COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS THE SOFTWARE 
* AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, 
* INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE 
* THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED 
* INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE 
* SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF SUCH HOLDER OR OTHER 
* PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* 
* 3. Interpretation of Sections 1 and 2.
* If the disclaimer of warranty and limitation of liability provided above cannot
* be given local legal effect according to their terms, reviewing courts shall apply
* local law that most closely approximates an absolute waiver of all civil liability
* in connection with the Software, unless a warranty or assumption of liability accompanies
* a copy of the Software in return for a fee.
*******************************************************************************/

#include <device.h>
#include <stdio.h>
#include <string.h>

#define BUFFER_LEN  64u

void main()
{
    uint16 count;
    uint8 buffer[BUFFER_LEN];
	uint16 rxTemp;
	uint8 Temp;
	uint16 n;
    
    /* Enable Global Interrupts */
    CyGlobalIntEnable;                        

    /* Start USBFS Operation with 3V operation */
    USBUART_1_Start(0u, USBUART_1_3V_OPERATION);

	/* Start UART */
	 UART_1_Start();     		/* Enabling the UART */

	/* Start LCD */
    LCD_Start();
    LCD_PrintString(" UART-USB-eSWiFi");
	
	UART_1_ClearRxBuffer(); 	/* Clear any bad data from WiFi module starup */
	
	P5_0_WiFiResetN_Write(1); 	/* Release reset on WiFi module */
	CyDelay(500);				/* Wait for WiFi module to startup */
    
    /* Wait for Device to enumerate */
    while(!USBUART_1_GetConfiguration());

    /* Enumeration is done, enable OUT endpoint for receive data from Host */
    USBUART_1_CDC_Init();

	/* simple USB to UART example */
    /* Main Loop: */
    for(;;)
    {
        while(USBUART_1_DataIsReady() != 0u)            /* Check for input data from PC */
        {   
            count = USBUART_1_GetAll(buffer);           /* Read received data and re-enable OUT endpoint */
            if(count != 0u)
            {
				for(n=0;n<count;n++)
				{
					UART_1_PutChar(buffer[n]);			/* Pass PC data to eS-WiFi Module */
				}
			}
		}
		while (USBUART_1_CDCIsReady() == 0u);    		/* Wait till component is ready to send more data to the PC */
		
		if (UART_1_GetRxBufferSize())					/* check if eS-WiFi data available */
		{
			rxTemp = UART_1_GetByte();					/* fetch eS-WiFi data */
			Temp = (int8) rxTemp;
			USBUART_1_PutData(&Temp, 1);       			/* Pass data to PC */
		}
    }   
}

/* [] END OF FILE */